#include <cmath>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_odeiv.h>
#include "caseconst.h"

int
  func (double t, const double y[], double f[],
	void *params)
{

   double R0   = *  (double *)params;
   double R1   = *( ((double *)params)+1 );
   double beta = *( ((double *)params)+2 );
   double mi   = *( ((double *)params)+3 );
   f[0] = 1.0;
   f[1] = (       ( -( R0 + (R1*exp((-beta)*t)) ) )   * (Q0/N0) * pow(y[1], 2.0) ) +
              ( ( -( mi -( R0 + (R1*exp((-beta)*t)) ) ) ) * y[1]                 );
   return GSL_SUCCESS;
}

int
  jac (double t, const double y[], double *dfdy,
       double dfdt[], void *params)
{

   double R0   = *  (double *)params;
   double R1   = *( ((double *)params)+1 );
   double beta = *( ((double *)params)+2 );
   double mi   = *( ((double *)params)+3 );
   gsl_matrix_view dfdy_mat
     = gsl_matrix_view_array (dfdy, 2, 2);
   gsl_matrix * m = &dfdy_mat.matrix;
   gsl_matrix_set (m, 0, 0, 0.0);
   gsl_matrix_set (m, 0, 1, 0.0);
   gsl_matrix_set (m, 1, 0, 0.0);
   gsl_matrix_set (m, 1, 1,
		      (       ( -( R0 + (R1*exp((-beta)*t)) ) )   * (Q0/N0) * (2.0) * y[1] ) +
		      ( ( -( mi -( R0 + (R1*exp((-beta)*t)) ) ) )                          )
		      );
   dfdt[0] = 0.0;
   dfdt[1] = ( (( beta)*R1*exp((-beta)*t)) * (Q0/N0) * pow(y[1], 2.0) ) +
                 ( ((-beta)*R1*exp((-beta)*t)) * y[1]                 );
   return GSL_SUCCESS;
}

int
  sim (double w[], int wle, double mu[])
{

   const gsl_odeiv_step_type * T
     = gsl_odeiv_step_rk4;

   gsl_odeiv_step * s
     = gsl_odeiv_step_alloc (T, 2);

   gsl_odeiv_system sys =
     {
	func, jac, 2, mu
     }
   ;

   double t = 0.0, t1 = wle-1;
   double h = 1;
   double y[2] =
     {
	0.0, 1.0
     }
   , y_err[2];
   double dydt_in[2], dydt_out[2];

   /* initialise dydt_in from system parameters */
   GSL_ODEIV_FN_EVAL(&sys, t, y, dydt_in);

   w[0] = 1;
   int i = 1;
   while (t < t1)
     {

	int status = gsl_odeiv_step_apply (s, t, h,
						      y, y_err,
						      dydt_in,
						      dydt_out,
						      &sys);

	if (status != GSL_SUCCESS)
	  break;

	dydt_in[0] = dydt_out[0];
	dydt_in[1] = dydt_out[1];

	t += h;

	//printf ("%.5e %.5e %.5e\n", t, y[0], y[1]);
	//cout << t << " " << y[0] << " " << y[1] << "\n";
	w[i] = y[1];
	i++;
     }


   gsl_odeiv_step_free (s);
   return 0;
}
